#include "etherxd.h"

/* connection management */
void manager_connect(thread t, int sock, struct sockaddr_in sa);
void manager_incoming(thread t, xmlnode node, int type);

void manager_init()
{
    xmlnode self, next;
    /* look at registry and start tlisteners on appropriate IPs */
    /* create entries for own IP's in connection arrays and initi arrays */
    /* return to main thread */

    debug("manager_init()");    

    self = xmlnode_get_child(app.config,"etherx");	    
    if(self != NULL)
    {           
        next = xmlnode_get_firstchild(self);
        while(next != NULL)
        {
            if(j_strcmp(xmlnode_get_name(next),"listen") == 0)
            {
                manager_init_listen(next);
            }
            next = xmlnode_get_nextsibling(next);
        }
    }
}

int manager_init_listen(xmlnode x)
{
    char *ip;
    xmlnode x2;

    debug("manager_init_listen()");
    x2 = xmlnode_get_firstchild(x);
    ip = xmlnode_get_data(x2);
    if(ip != NULL)
    {
        log(LOG_DEBUG,"etherx_manager_init_listen",ip);
        tlisten_new(5269,ip,manager_connect,NULL);
            return 0;
    }
    return 1;
}

/* this is the callback on the listener thread whenever there is a new connect */
void manager_connect(thread t, int sock, struct sockaddr_in sa)
{
    thread new;
    printf("connection established (fd: %d, ip: %s, port: %d)\n",
                sock, inet_ntoa(sa.sin_addr), ntohs(sa.sin_port));
    new = xmlstream_new(sock, (xmlstream_onNode)manager_onNode);
}

/* this is the callback on the xmlstream thread whenever there is a new node available */
void manager_onNode(thread t, xmlnode in, int type)
{
    xmlnode x, next, config;
    tstream ts = (tstream)(t->data);
    char hello[] = "<stream xmlns='http://jabber.org/xmlns/jabber#19991001' xmlns:etherx='http://jabber.org/xmlns/etherx#19991001' from=''>";
    char *name;
    static int key = 1;
    char *id;

    x = xmlnode_dup(in);

    printf("XML NODE: %d type[%d] '%s'\n",ts->fd,type,xmlnode2str(x));
    switch(type)
    {
        case XMLSTREAM_NODE:
            name = xmlnode_get_name(x);
            if(j_strcmp(name,"etherx:transport") == 0)
            {
                /* check the given id and secret in our config, if match, secure this stream as a transport */

                /* XXX kludge for now, store the id on the thread stream */
                id = pmalloc_key(t->p, (int)&key, xmlnode_get_datasz(xmlnode_get_firstchild(x)) + 1);
                strcpy(id,xmlnode_get_data(xmlnode_get_firstchild(x)));
                printf("got etherx:transport id=%s [id:%d key:%d]\n",id,id,&key);
            }
            if(j_strcmp(name,"etherx:registry") == 0 && j_strcmp(xmlnode_get_attrib(x,"type"),"request") == 0)
            {
                /* they asked for their config, give it to em :) */
                id = pmalloc_key(t->p, (int)&key, 0); /* get this transports id, assume it's there since we're still kludging :) */
                printf("got etherx:registry for id=%s [id:%d key:%d]\n",id,id,&key);

                next = xmlnode_get_firstchild(app.config);
                config = NULL;
                while(next != NULL)
                {
                    if(j_strcmp(xmlnode_get_name(next),"transport") == 0 && j_strcmp(xmlnode_get_attrib(next,"id"),id) == 0)
                        config = xmlnode_dup(next);
                    next = xmlnode_get_nextsibling(next);
                }

                debug("sending config data");
                if(config != NULL)
                {
                    /* XXX hackery in progress!  just plunk the config section down the line */
                    tstream_write(t, "<etherx:registry>", 17);
                    xmlstream_write(t, config);
                    tstream_write(t, "</etherx:registry>", 18);
                }else{
                    tstream_write(t, "<etherx:registry/>", 18);
                }
            }
            break;
        case XMLSTREAM_ROOT:
            tstream_write(t, hello, strlen(hello));
            break;
        case XMLSTREAM_ERROR:
            break;
    }
}

/* this is the callback that individual DSO loaded transports call to deliver epackets to etherx */
void manager_onDSO(epacket e)
{
    printf("XML NODE: dso '%s'\n",xmlnode2str(e->x));
}

