#include "jabber/jlib.h"
#include "jabber/etherx.h"

thread _etherx_thread = NULL;
void (*_etherx_receive_handler)(epacket e, int flag);

/* only called by socket connections, starts xmlstream to etherxd, returns success */
int etherx_init(char *id, char *secret, char *host, void (*receive)(epacket e, int flag))
{
    int s;
    xmlnode x;
    char hello[] = "<stream xmlns='http://jabber.org/xmlns/jabber.19991001' xmlns:etherx='http://jabber.org/xmlns/etherx.19991001' to='' from=''>";

    /* so that etherx_init can be rerun internally and it will remember the orig args from the app */
    static char *s_id = NULL;
    static char *s_secret = NULL;
    static char *s_host = NULL;

    if(s_id == NULL)
        s_id = j_strdup(id);
    if(s_secret == NULL)
        s_secret = j_strdup(secret);
    if(s_host == NULL)
        s_host = j_strdup(host);
    if(s_host == NULL)
        s_host = j_strdup("localhost");

    _etherx_receive_handler = receive;

    s = make_netsocket(5269,s_host,NETSOCKET_CLIENT);
    if(s <= 0)
    {
        log(LOG_ERROR,"etherx_connect_failure",s_host);
        return -1;
    }

    _etherx_thread = xmlstream_new(s, (xmlstream_onNode)_etherx_receive);
    printf("_etherx_thread[%d] (new)\n",_etherx_thread);
    tstream_write(_etherx_thread, hello, strlen(hello));

    x = xmlnode_new_tag("etherx:transport");
    xmlnode_put_attrib(x, "secret", s_secret);
    xmlnode_insert_cdata(x, s_id, strlen(s_id));
    xmlstream_write(_etherx_thread,x);

    x = xmlnode_new_tag("etherx:registry");
    xmlnode_put_attrib(x, "type", "request");
    xmlstream_write(_etherx_thread,x);

    (*receive)(NULL, EPACKET_INIT); /* tell the transport to start up! */

    return 1;
}

void _etherx_receive(thread t, xmlnode x, int flag)
{
    epacket e;

    switch(flag)
    {
        case XMLSTREAM_ROOT:
            debug("got root tag from etherx!");
            /* ignore the root tag, transports don't use it */
            break;
        case XMLSTREAM_NODE:
            debug("got node from etherx!");
            if(j_strcmp(xmlnode_get_name(x), "etherx:registry") == 0)
            {
                e = epacket_new(xmlnode_get_firstchild(x), NULL, NULL);
                (*_etherx_receive_handler)(e, EPACKET_REGISTRY);
            }else{
                e = epacket_new(x, NULL, NULL);
                (*_etherx_receive_handler)(e, EPACKET_NORMAL);
            }
            break;
    }
}

void _etherx_cleanup(void *arg)
{
    /* the connection died, reconnect? */
    log(LOG_WARN,"libetherx","Connection to etherxd failed, attempting to reconnect...");
}
