#include "jabber/jlib.h"
#ifndef WIN32

/* socket.c
 *
 * Simple wrapper to make socket creation easy.
 * type = NETSOCKET_SERVER is local listening socket
 * type = NETSOCKET_CLIENT is connection socket
 * type = NETSOCKET_UDP
 */

int make_netsocket(u_short port, char *host, int type)
{
	int s, flag = 1;
	struct sockaddr_in sa;
	struct in_addr *saddr;
	int socket_type;

        log(LOG_DEBUG,"make_netsocket",host);

	/* is this a UDP socket or a TCP socket? */
	socket_type = (type == NETSOCKET_UDP)?SOCK_DGRAM:SOCK_STREAM;

	bzero((void *)&sa,sizeof(struct sockaddr_in));

	if((s = socket(AF_INET,socket_type,0)) < 0)
		return(-1);
	if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (char*)&flag, sizeof(flag)) < 0)
		return(-1);

	saddr = make_addr(host);
	if(saddr == NULL)
		return(-1);
	sa.sin_family = AF_INET;
	sa.sin_port = htons(port);

	if(type == NETSOCKET_SERVER)
	{
		/* bind to specific address if specified */
		if(host != NULL)
			sa.sin_addr.s_addr = saddr->s_addr;

		if(bind(s,(struct sockaddr*)&sa,sizeof sa) < 0)
		{
			close(s);
			return(-1);
		}
	}
	if(type == NETSOCKET_CLIENT)
	{
		sa.sin_addr.s_addr = saddr->s_addr;
		if(connect(s,(struct sockaddr*)&sa,sizeof sa) < 0)
		{
			close(s);
			return(-1);
		}
	}
	if(type == NETSOCKET_UDP)
	{
		/* bind to all addresses for now */
		if(bind(s,(struct sockaddr*)&sa,sizeof sa) < 0)
		{
			close(s);
			return(-1);
		}

		/* specify default recipient for read/write */
		sa.sin_addr.s_addr = saddr->s_addr;
		if(connect(s,(struct sockaddr*)&sa,sizeof sa) < 0)
		{
			close(s);
			return(-1);
		}
	}


	return(s);
}


struct in_addr *make_addr(char *host)
{
	struct hostent *hp;
	static struct in_addr addr;
	char myname[MAXHOSTNAMELEN + 1];

	if(host == NULL || strlen(host) == 0)
	{
		gethostname(myname,MAXHOSTNAMELEN);
		hp = gethostbyname(myname);
		if(hp != NULL)
		{
			return (struct in_addr *) *hp->h_addr_list;
		}
	}else{
		addr.s_addr = inet_addr(host);
		if(addr.s_addr != -1)
		{
			return &addr;
		}
		hp = gethostbyname(host);
		if(hp != NULL)
		{
			return (struct in_addr *) *hp->h_addr_list;
		}
	}
	return NULL;
}

#endif
