#include "jabber/jlib.h"

thread xmlstream_new(int fd, xmlstream_onNode evt)
{
    log(LOG_DEBUG,"xmlstream_new",NULL);
    return tstream_new(fd, _xmlstream_main, evt);
}


int xmlstream_write(thread t, xmlnode x)
{
    char *buff;
    int len;

    log(LOG_DEBUG,"xmlstream_write",NULL);

    if(t == NULL || x == NULL)
        return 0;

    /* serialize xmlnode to a string */
    buff = xmlnode2str(x);
    tstream_write(t, buff, strlen(buff));

    return 0;
}


void _xmlstream_startElement(void* userdata, const char* name, const char** atts)
{
    thread t = (thread)userdata;
    tstream ts = (tstream)(t->data);
    xmlnode current = (xmlnode)(ts->data_aux2);
    xmlstream_onNode onNode = (xmlstream_onNode)(ts->read_evt_data);

    log(LOG_INSANE,"stream_startElement",name);

    if(current == NULL)
    {
        log(LOG_INSANE,"stream_startElement","creating new node");
        current = xmlnode_new_tag(name);
        xmlnode_put_expat_attribs(current, atts);

        if(ts->data_aux3 == NULL)
        { /* this is a special flag saying that this is the <stream> root */
            log(LOG_INSANE,"stream_startElement","processing root node");
            ts->data_aux3 = (void *)1;
            (onNode)(t, current, XMLSTREAM_ROOT);
            xmlnode_free(current);
            current = NULL;
        }
    }
    else
    {
        log(LOG_INSANE,"startElement","creating child node");
        current = xmlnode_insert_tag(current, name);
        xmlnode_put_expat_attribs(current, atts);
    }

    ts->data_aux2 = current;
}


void _xmlstream_endElement(void* userdata, const char* name)
{
    thread t = (thread)userdata;
    tstream ts = (tstream)(t->data);
    xmlnode current = (xmlnode)(ts->data_aux2);
    xmlstream_onNode onNode = (xmlstream_onNode)(ts->read_evt_data);
    xmlnode parent;

    log(LOG_INSANE,"stream_endElement",name);

    /* if it's already NULL we've received </stream>, tell the app and we're outta here */
    if(current == NULL)
    {
        (onNode)(t, current, XMLSTREAM_CLOSE);
        pth_exit(NULL);
    }else{
        parent = xmlnode_get_parent(current);
        if(parent == NULL)
        { /* we are the top-most node, feed to the app and delete */
            (onNode)(t, current, XMLSTREAM_NODE);
            xmlnode_free(current);
        }
        ts->data_aux2 = parent;
    }
}


void _xmlstream_charData(void* userdata, const char* s, int len)
{
    thread t = (thread)userdata;
    tstream ts = (tstream)(t->data);
    xmlnode current = (xmlnode)(ts->data_aux2);
    xmlstream_onNode onNode = (xmlstream_onNode)(ts->read_evt_data);

    if(current == NULL)
    {
        /* we must be in the root of the stream where CDATA is irrelevant */
        return;
    }

    log(LOG_INSANE,"charData",s);
    xmlnode_insert_cdata(current, s, len);
}


int _xmlstream_main(void *arg, void *buffer, int bufsz)
{
    /*
    this is called when there is new data on the tstream
    it should process the XML using the special stream expat callbacks which
    call (ts->read_evt_data)()
    */

    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    xmlstream_onNode onNode = (xmlstream_onNode)ts->read_evt_data;
    XML_Parser p;

    log(LOG_DEBUG,"_xmlstream_main",NULL);

    p = _xmlstream_fetchParser(t);

    log(LOG_INSANE,"XML_Parse",buffer);

    if(!XML_Parse(p, buffer, bufsz, 0))
    {
        log(LOG_ERROR,"xmlstream_parse_error",(char *)XML_ErrorString(XML_GetErrorCode(p)));
        (onNode)(t, NULL, XMLSTREAM_ERROR);
        pth_exit(NULL);
    }
}

XML_Parser _xmlstream_fetchParser(thread t)
{
    tstream ts = (tstream)(t->data);
    XML_Parser p = (XML_Parser)(ts->data_aux1);

    if(p == NULL)
    {
        log(LOG_INSANE,"_xmlstream_fetchParser","init");
        p = XML_ParserCreate(NULL);
        pth_cleanup_push(_xmlstream_cleanup, (void *)t);
        ts->data_aux1 = (void *)p;
        XML_SetUserData(p, (void *)t);
        XML_SetElementHandler(p, (void *)_xmlstream_startElement, (void *)_xmlstream_endElement);
        XML_SetCharacterDataHandler(p, (void *)_xmlstream_charData);
    }

    return p;
}

void _xmlstream_cleanup(void *arg)
{
    thread t = (thread)arg;
    tstream ts = (tstream)(t->data);
    xmlnode *x = (xmlnode *)(ts->data_aux2);
    XML_Parser p = (XML_Parser)(ts->data_aux1);

    log(LOG_DEBUG,"_xmlstream_cleanup",NULL);
    XML_ParserFree(p);
}