
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include "jlib.h"

xmlnode nodes;
xmlnode current_node;


void printnode(xmlnode n, int tabcnt)
{
     int i;
     if (n == NULL)
	  return;
     for (i = 0; i < tabcnt; i++)
	  putchar('\t');
     switch(xmlnode_get_type(n))
     {
     case NTYPE_ATTRIB: printf("Attribute: %s = %s\n", xmlnode_get_name(n), xmlnode_get_data(n)); break;
     case NTYPE_TAG:    printf("Tag: [%s]\n", xmlnode_get_name(n)); break;
     case NTYPE_CDATA:  printf("CDATA: [%s]\n", xmlnode_get_data(n)); break;
     }

     if (xmlnode_has_attribs(n))
	  printnode(xmlnode_get_firstattrib(n), tabcnt +1);

     if (xmlnode_has_children(n))
	  printnode(xmlnode_get_firstchild(n), tabcnt+1);

     printnode(xmlnode_get_nextsibling(n), tabcnt);
}


void expat_startElement(void* userdata, const char* name, const char** atts)
{
     if (nodes == NULL)
     {
	  /* allocate a base node */
	  nodes = xmlnode_new_tag(name);
	  current_node = nodes;
     }
     else
     {
	  current_node = xmlnode_insert_tag(current_node, name);
     }
}

void expat_endElement(void* userdata, const char* name)
{
     if (strcmp(name, xmlnode_get_name(current_node)) == 0)
	  current_node = xmlnode_get_parent(current_node);
     else
	  printf("Parse error: %s", name);
}

void expat_charData(void* userdata, const char* s, int len)
{
     xmlnode_insert_cdata(current_node, s, len);
}
     

int main (int argc, char** argv)
{
     int         fd;
     XML_Parser  p;
     char        buf[200];
     int         len;
     int         done;

     /* Open a XML file
     open("test.xml", O_RDONLY); */

     fd = stdin;
     
     /* Initialize xmlnodes */
     nodes = NULL;
     current_node = NULL;

     /* Initialize expat */
     p = XML_ParserCreate(NULL);
     XML_SetUserData(p, p);
     XML_SetElementHandler(p, expat_startElement, expat_endElement);
     XML_SetCharacterDataHandler(p, expat_charData);

     /* Loop through the file, parsing data */
     do
     {
          /* Read a chunk */
          len = fread(buf, 1, sizeof(buf), stdin);
          done = len < sizeof(buf);

	  /* Parse it */
	  if (!XML_Parse(p, buf, len, 0))
	  {
            fprintf(stderr,"%s at line %d\n",
                XML_ErrorString(XML_GetErrorCode(p)),
                XML_GetCurrentLineNumber(p));
	       return -1;
	  }
     } while (!done);
     
     /* Print resulting xmlnodes */
     printnode(nodes, 0);

     close(fd);
}
