#include "jserver.h"

/* start all system threads here, init any globals */
void config_startup(void)
{
    debug("JSERVER STARTUP!");
}

/* this is where everything is configured */
void config_handle(xmlnode x)
{
    xmlnode next, l, m;

    debug("config_handle");

    /* we got the chunk from the config file, 
    make sure we init everything depending on config reguardless if they were configured  */
    next = xmlnode_get_firstchild(x);
    l = m = NULL;
    while(next != NULL)
    {
        if(j_strcmp(xmlnode_get_name(next),"name") == 0)
            config_names(CMD_SET,xmlnode_get_data(xmlnode_get_firstchild(next)));
        if(j_strcmp(xmlnode_get_name(next),"transport") == 0)
            config_transports(CMD_SET,next);
        if(j_strcmp(xmlnode_get_name(next),"service") == 0)
            config_service(next);
        if(j_strcmp(xmlnode_get_name(next),"log") == 0)
            l = next;
        if(j_strcmp(xmlnode_get_name(next),"modules") == 0)
            m = next;
        next = xmlnode_get_nextsibling(next);
    }

    logger_init(l);
    modules_init(m);
}

struct names
{
    char *name;
    struct names *next;
};

/* this is a special internal name resolver for the dns name(s) of this transport */
char *config_names(int cmd, char *name)
{
    static struct names *n = NULL, *get = NULL;
    struct names *new;
    
    switch(cmd)
    {
        case CMD_SET: /* only intended to be run at config time */
            if(config_names(CMD_CHECK,name) != NULL)
                return NULL;
            new = malloc(sizeof(struct names));
            new->name = j_strdup(name);
            new->next = n;
            n = new;
            break;
        case CMD_CHECK:
            new = n;
            while(new != NULL)
            {
                if(j_strcmp(name,new->name) == 0)
                    return new->name;
                new = new->next;
            }
            break;
        case CMD_GET: /* loops through name list, returns NULL at end */
            if(get == NULL)
                get = n;
            else
                get = get->next;
            if(get != NULL)
                return get->name;
            break;
    }
    return NULL;
}

struct transports
{
    char *id;
    char *name;
    struct transports *next;
};

/* 
this is a container to store the configured transports available for users
*/
char *config_transports(int cmd, xmlnode x)
{
    static struct transports *t = NULL, *get = NULL;
    struct transports *new;
/*    
    switch(cmd)
    {
        case CMD_SET:
            if(config_transports(CMD_CHECK,name) != NULL)
                return NULL;
            new = malloc(sizeof(struct names));
            new->name = j_strdup(name);
            new->next = t;
            t = new;
            break;
        case CMD_CHECK:
            new = n;
            while(new != NULL)
            {
                if(j_strcmp(name,new->name) == 0)
                    return new->name;
                new = new->next;
            }
            break;
        case CMD_GET:
            if(get == NULL)
                get = n;
            else
                get = get->next;
            if(get != NULL)
                return get->name;
            break;
    }
*/
    return NULL;
}

void config_service(xmlnode x)
{
    /* dso load the service and initialize it */
    char *dso;

    dso = xmlnode_get_attrib(x, "dso");
    dso_load(dso, "init", (void *)x);
}

