#include "jserver.h"

resource resources = NULL;

resource resource_create(int type, char *user, char *res, pth_msgport_t handler)
{
    resource new;

    new = malloc(sizeof(_resource));
    new->type = type;
    new->user = j_strdup(user);
    new->res = j_strdup(res);
    new->handler = handler;
    new->next = resources;
    resources = new;
}

resource resource_lookup(char *user, char *res)
{
    resource next;

    next = resources;
    while(next != NULL)
    {
        if(j_strcmp(user, next->user) == 0 && j_strcmp(res, next->res) == 0)
            return next;
        next = next->next;
    }
    return NULL;
}

void resource_free(resource r)
{
    resource next;

    if(r == NULL)
        return;
    
    if(resources == r)
    {
        resources = r->next;
    }else{
        next = resources;
        while(next != NULL)
        {
            if(next->next == r)
            {
                next->next = next->next->next;
                break;
            }
            next = next->next;
        }
    }

    free(r->user);
    free(r->res);
    free(r);
}

void deliver(jpacket p)
{
    resource lookup;
    static pth_msgport_t unknown_mp = NULL;
    static pth_msgport_t external_mp = NULL;
    
    if(unknown_mp == NULL)
        unknown_mp = pth_msgport_find("unknown");
    if(external_mp == NULL)
        external_mp = pth_msgport_find("external");

    /* important main function, takes any packet and attempts to deliver it to the correct session/thread */
    /* calls resource_lookup, creates/sends message, etc */

    if(p->to == NULL) /* invalid */
        return;

    /* first, is it destined externally? */
    if(config_names(CMD_CHECK,p->to->server) == NULL)
    {
        pth_msgport_put(external_mp,p);
        return;
    }

    /* is this for a known resource? */
    lookup = resource_lookup(p->to->user, p->to->resource);

    /* unknown resources handled by unknown thread */
    if(lookup == NULL)
    {
        pth_msgport_put(unknown_mp,p);
        return;
    }

    pth_msgport_put(lookup->handler,p);
}

